from django.shortcuts import render_to_response

from datetime import datetime, date, timedelta
import calendar
import icalendar

def first_day_of_month(dt, d_years=0, d_months=0):
    y, m = dt.year + d_years, dt.month + d_months
    a, m = divmod(m-1, 12)
    return datetime(y+a, m+1, 1)

def last_day_of_month(dt):
    return first_day_of_month(dt, 0, 1) + timedelta(-1)

def index(request):
	today = datetime.today()
	cal_begin, cal_end = first_day_of_month(today), last_day_of_month(today)

	cal = icalendar.Calendar.from_string(open('/home/arnaud/ib.ics','rb').read())

	events = []
	for component in cal.walk():
		if component.name == 'VEVENT':
			dtstart = component.decoded('dtstart')
			dtend = component.decoded('dtend')

			if cal_begin <= dtstart <= cal_end or cal_begin <= dtend <= cal_end:
				events.append({'start': dtstart, 'end': dtend,
							   'summary': component['summary'],
							   'description': component.get('description'),
							   'url': component.get('url') })
			return render_to_response('pcalendar/month.html',
									  { 'events': events })

def month(request, year=None, month=None):
	today = datetime.today()
	if year:
		year = int(year)
	else:
		year = today.year
	if month:
		month = int(month)
	else:
		month = today.month

	cal = calendar.monthcalendar(year, month)

	ical = icalendar.Calendar.from_string(open('/home/arnaud/ib.ics','rb').read())
	events = {}
	all_day_events = {}
	for component in ical.walk():
		if component.name == 'VEVENT':
			dtstart = component.decoded('dtstart')
			dtend = component.decoded('dtend')

			component['dtstart'] = dtstart
			component['dtend'] = dtend

			if type(dtstart) == date:
				dico = all_day_events
				key = dtstart
			else:
				dico = events
				key = dtstart.date()

			if dico.has_key(key):
				dico[key].append(component)
			else:
				dico[key] = [component]

	for week in cal:
		for i in range(len(week)):
			if week[i]:
				day = date(year, month, week[i])
				week[i] = { 'date': day }
				if events.has_key(day):
					events[day].sort(lambda e1, e2: cmp(e1['dtstart'], e2['dtstart']))
					week[i]['events'] = events[day]
				if all_day_events.has_key(day):
					week[i]['all_day_events'] = all_day_events[day]
			else:
				week[i] = None

	return render_to_response('pcalendar/month.html',
							  { 'calendar': cal, 'today': today })
