# -*- coding: utf-8 -*-
from django.http import Http404
import smtplib
from email.MIMEText import MIMEText
from django import newforms as forms
from log import *

# moins consomateur que le get_object_or_404 standard
def get_object_or_404(klass, *args, **kwargs):
    try:
        return klass.objects.get(*args, **kwargs)
    except klass.DoesNotExist:
        raise Http404


def send_form(form, sender, recipients, subject):
    "Send django form content by mail"
    message = ""
    for field in form:
        message += '%s : %s\n' % (field.label, field.data)
    
    mail = MIMEText(message, _charset='utf-8')
    mail['Subject'] = subject
    mail['From'] = sender
    mail['To'] = recipients
    s = smtplib.SMTP()
    s.connect()
    s.sendmail(sender, recipients.split(','), mail.as_string())
    s.close()

    
def get_permission(instance, user):
    "Pour l'instant, on ne considère que les permissions de la table (hors donnees volatiles)"
    if user.is_superuser:
        return 'all'
    role = instance.get_role(user)
    return instance.permission['table'][role]


def modify(instance, fields = None):
    Class = instance.__class__
    object_ref = Class.objects.filter(ref_object=instance.uid)
    if not len(object_ref):
        values = instance.__dict__.copy()
        del values['uid']
        object_ref = Class(**values)
        object_ref.ref_object = instance.uid
        object_ref.save()
    else:
        uid_ref = object_ref[0].uid
        object.ref_object = instance.uid
        instance.uid = uid_ref

def validate(instance, fields = None):
    Class = instance.__class__
    instance.save()
    for instance_ref in Class.instances.filter(ref_object=instance.uid):
        instance_ref.delete()

    


class DataForm(forms.Form):
    def __init__(self, instance, data=None, auto_id='id_%s', prefix=None):
        self.instance = instance
        Class = instance.__class__
        instance_ref = Class.objects.filter(ref_object=instance.uid)
        if len(instance_ref):
            initial = instance_ref[0].__dict__
            for key, field in self.fields.items():
                valid_value = getattr(instance, key)
                if initial[key] != valid_value:
                    field.help_text = getattr(instance, key)
        else:
            initial = instance.__dict__
        forms.Form.__init__(self, data, auto_id, prefix, initial)
        

    # from django/newforms/models.py
    def save(self, user, fields=None):
        """
        Saves bound Form ``form``'s clean_data into model instance ``instance``.

        Assumes ``form`` has a field for every non-AutoField database field in
        ``instance``. If commit=True, then the changes to ``instance`` will be
        saved to the database. Returns ``instance``.
        """
        from django.db import models
        opts = self.instance.__class__._meta
        if self.errors:
            raise ValueError("The %s could not be saved because the data didn't validate." % opts.object_name)
        clean_data = self.clean_data
        for f in opts.fields:
            if not f.editable or isinstance(f, models.AutoField) or not f.name in clean_data:
                continue
            if fields and f.name not in fields:
                continue
            setattr(self.instance, f.name, clean_data[f.name])


        permission = get_permission(self.instance, user)
        if permission in ('all', 'save'):
            validate(self.instance)
        elif permission == 'modify':
            modify(self.instance)
        else:
            raise ValueError("You don't have permission to save %s object." % opts.object_name)

        # ?????????
    #     for f in opts.many_to_many:
    #         if fields and f.name not in fields:
    #             continue
    #         if f.name in clean_data:
    #             setattr(instance, f.attname, clean_data[f.name])
        # GOTCHA: If many-to-many data is given and commit=False, the many-to-many
        # data will be lost. This happens because a many-to-many options cannot be
        # set on an object until after it's saved. Maybe we should raise an
        # exception in that case.
        return self.instance
