"""
PostgreSQL database backend for Django.

Requires psycopg 2: http://initd.org/projects/psycopg2
"""

from django.db.backends.postgresql_psycopg2.base import *
from django.db.backends import util
postgresql_psycopg2DatabaseWrapper = DatabaseWrapper
import django.pimentech.log as log
from time import time
import re

r_pref_sql = re.compile(r'^SELECT|^select|^UPDATE|^update|^INSERT|^insert')


class CursorDebugWrapper(util.CursorDebugWrapper):

    def execute(self, sql, params=()):
        start = time()
        if r_pref_sql.match(sql):
            self.cursor.execute("EXPLAIN " + sql, params)
            plan = self.cursor.fetchall()[0][0]
        else:
            plan = ''
        try:
            return self.cursor.execute(sql, params)
        finally:
            stop = time()
            # If params was a list, convert it to a tuple, because string
            # formatting with '%' only works with tuples or dicts.
            if not isinstance(params, (tuple, dict)):
                params = tuple(params)
            #plan = ''
            sql = sql % params
            sql = sql.replace(",", ", ")
            self.db.queries.append({
                'sql': sql,
                'time': "%.3f %s" % (stop - start, plan),
            })



class DatabaseWrapper(postgresql_psycopg2DatabaseWrapper):

    def cursor(self):
        from django.conf import settings
        set_tz = False
        if self.connection is None:
            set_tz = True
            if settings.DATABASE_NAME == '':
                from django.core.exceptions import ImproperlyConfigured
                raise ImproperlyConfigured, "You need to specify DATABASE_NAME in your Django settings file."
            conn_string = "dbname=%s" % settings.DATABASE_NAME
            if settings.DATABASE_USER:
                conn_string = "user=%s %s" % (settings.DATABASE_USER, conn_string)
            if settings.DATABASE_PASSWORD:
                conn_string += " password='%s'" % settings.DATABASE_PASSWORD
            if settings.DATABASE_HOST:
                conn_string += " host=%s" % settings.DATABASE_HOST
            if settings.DATABASE_PORT:
                conn_string += " port=%s" % settings.DATABASE_PORT
            self.connection = Database.connect(conn_string, **self.options)
            self.connection.set_isolation_level(1) # make transactions transparent to all cursors
        cursor = self.connection.cursor()
        cursor.tzinfo_factory = None
        if set_tz:
            cursor.execute("SET TIME ZONE %s", [settings.TIME_ZONE])
        if settings.DEBUG:
            return CursorDebugWrapper(cursor, self)
        return cursor

