def model_to_dict(instance, fields=None, exclude=None):
	"""
	Returns a dict containing the data in ``instance`` suitable for passing as
	a Form's ``initial`` keyword argument.

	``fields`` is an optional list of field names. If provided, only the named
	fields will be included in the returned dict.

	``exclude`` is an optional list of field names. If provided, the named
	fields will be excluded from the returned dict, even if they are listed in
	the ``fields`` argument.
	"""
	# avoid a circular import
	from django.db.models.fields.related import ManyToManyField
	opts = instance._meta
	data = {}
	for f in opts.fields + opts.many_to_many:
		if not f.editable:
			continue
		if fields and not f.name in fields:
			continue
		if exclude and f.name in exclude:
			continue
		if isinstance(f, ManyToManyField):
			# If the object doesn't have a primry key yet, just use an empty
			# list for its m2m fields. Calling f.value_from_object will raise
			# an exception.
			if instance._get_pk_val() is None:
				data[f.name] = []
			else:
				# MultipleChoiceWidget needs a list of pks, not object instances.
				data[f.name] = [obj._get_pk_val() for obj in f.value_from_object(instance)]
		else:
			data[f.name] = f.value_from_object(instance)
	return data


