# -*- coding: utf-8 -*-

import htmlentitydefs

from django import template
from django.utils import translation
from django.newforms.util import smart_unicode

register = template.Library()

from types import *

def get_or_getattr(x, key, default=None):
	if type(x) is DictType:
		return x.get(key, default)
	else:
		return getattr(x, key, default)

def trad(objet, attribut, default='', french=True):
	langue = translation.get_language()[:2]
	if (default == True or default == "True"):
		default = str(objet)
	if (french == True or french == "True"):
		french_trad = get_or_getattr(objet, '%s_fr' % attribut, None)
	else:
		french_trad = None
	return get_or_getattr(objet, '%s_%s' % (attribut, langue), None) \
		   or french_trad or get_or_getattr(objet, '%s' % attribut, None) \
		   or default
trad = register.simple_tag(trad)

def list_of_links(value):
	return '\n'.join(['<li><a href="%s">%s</a></li>' % (href, title) for (title, href) in value])
list_of_links = register.filter(list_of_links)

def antispamify(value):
	return value.replace('@', ' &#x0040; ').replace('.', ' &#x002e; ')
antispamify = register.filter(antispamify)

def pages_navigation(nb_results, page, nb_per_page=15, append_get=None):
	import math
	nb_pages = int(math.ceil(float(nb_results)/nb_per_page))
	prev_pages = range(1, page)[-4:]
	next_pages = range(page + 1, nb_pages + 1)[:4]
	return { 'page': page,
			 'prev_pages': prev_pages,
			 'prev_page': prev_pages and prev_pages[-1],
			 'next_pages': next_pages,
			 'next_page': next_pages and next_pages[0],
			 'nb_pages': nb_pages,
			 'append_get' : append_get and append_get + '&' or '' }
pages_navigation = register.inclusion_tag('common/pages_navigation.html')(pages_navigation)

def html_entities(value, include_essential=False):
	"""
	Transforms non ASCII caracters into HTML entities
	"""
	ret = ''
	for char in smart_unicode(value):
		char_ord = ord(char)
		if char_ord in htmlentitydefs.codepoint2name and (include_essential or char not in ('<', '>','"',"'", '&')):
			ret += '&' + htmlentitydefs.codepoint2name[char_ord] + ';'
		else:
			ret = ret + char
	return ret
register.filter('html_entities', html_entities)

def trim_string(value, limit):
	return value[:limit] + (len(value) >= limit and '...' or '')
register.filter('trim_string', trim_string)

def fourchette(min, max, value=None, label=""):
	if min and max:
		return '%g&ndash;%g %s' % (min, max, label)
	elif value or min or max:
		return '%g %s' % (value or min or max, label)
	else:
		return ''
fourchette = register.simple_tag(fourchette)

def telephone(value):
	import re
	return re.sub(r'([0-9]{2})[.-]?', r'\1 ', value).strip()
register.filter('telephone', telephone)
