from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse
from django.contrib.admin.views.main import *
from django.utils.datastructures import MultiValueDict
from django.db import transaction
from django.contrib.admin.views import main
from django import newforms as forms
from django.db import connection

from django.utils.translation import ugettext as _

import copy
import csv
import cStringIO

def safe_save(instance):
	for field in instance._meta.fields:
		name = field.name
		if hasattr(instance, name):
			value = getattr(instance, name)
			typ = field.__class__.__name__
			if typ == 'IntegerField':
				if value == '':
					setattr(instance, field.name, None)



def pim_add_stage(request, app_label, model_name, show_delete=False, form_url='', post_url=None, post_url_continue='../%s/', object_id_override=None):
	if request.POST:
		for name, value in request.POST.items():
			if name.endswith('modification_auth_user') or name.endswith('creation_auth_user'):
				MultiValueDict.__setitem__(request.POST, name, '%s' % request.user.id)
	return add_stage(request, app_label, model_name, show_delete, form_url, post_url, post_url_continue, object_id_override)

def pim_change_stage(request, app_label, model_name, object_id):
	if request.POST:
		for name, value in request.POST.items():
			if name.endswith('modification_auth_user') or \
				   (name.endswith('creation_auth_user') and not request.POST[name]):
				MultiValueDict.__setitem__(request.POST, name, '%s' % request.user.id)
	return change_stage(request, app_label, model_name, object_id)


def view(request, app_label, model_name, object_id):
	model = models.get_model(app_label, model_name)
	object_id = unquote(object_id)
	if model is None:
		raise Http404("App %r, model %r, not found" % (app_label, model_name))
	opts = model._meta
	field_sets = opts.admin.get_field_sets(opts)

	try:
		manipulator = model.ChangeManipulator(object_id)
	except model.DoesNotExist:
		raise Http404('%s object with primary key %r does not exist' % (model_name, escape(object_id)))

	# Populate new_data with a "flattened" version of the current data.
	new_data = manipulator.flatten_data()

	id_order_list = []
	for rel_obj in opts.get_ordered_objects():
		id_order_list.extend(getattr(manipulator.original_object, 'get_%s_order' % rel_obj.object_name.lower())())
		if id_order_list:
			new_data['order_'] = ','.join(map(str, id_order_list))
			errors = {}

	# Populate the FormWrapper.
	errors = {}
	form = oldforms.FormWrapper(manipulator, new_data, errors)
	form.original = manipulator.original_object
	form.order_objects = []

	#TODO Should be done in flatten_data  / FormWrapper construction
	for related in opts.get_followed_related_objects():
		wrt = related.opts.order_with_respect_to
		if wrt and wrt.rel and wrt.rel.to == opts:
			func = getattr(manipulator.original_object, 'get_%s_list' %
					related.get_accessor_name())
			orig_list = func()
			form.order_objects.extend(orig_list)

	c = template.RequestContext(request, {
		'server_name': request.META['SERVER_NAME'],
		'path': request.path,
		'title': _('Change %s') % opts.verbose_name,
		'form': form,
		'object_id': object_id,
		'original': manipulator.original_object,
		'is_popup': request.REQUEST.has_key('_popup'),
	})
	#return render_change_form(model, manipulator, c, change=True)
	#def render_change_form(model, manipulator, context, add=False, change=False, form_url=''):

	auto_populated_fields = [f for f in opts.fields if f.prepopulate_from]
	field_sets = opts.admin.get_field_sets(opts)
	original = getattr(manipulator, 'original_object', None)
	bound_field_sets = [field_set.bind(c['form'], original, AdminBoundFieldSet) for field_set in field_sets]
	first_form_field_id = bound_field_sets[0].bound_field_lines[0].bound_fields[0].form_fields[0].get_id();
	ordered_objects = opts.get_ordered_objects()
	inline_related_objects = opts.get_followed_related_objects(manipulator.follow)
	extra_context = {
		'add': False,
		'change': False,
		'has_delete_permission': False,
		'has_change_permission': False,
		'has_file_field': False,
		'has_absolute_url': hasattr(model, 'get_absolute_url'),
		'auto_populated_fields': auto_populated_fields,
		'bound_field_sets': bound_field_sets,
		'first_form_field_id': first_form_field_id,
		'javascript_imports': get_javascript_imports(opts, auto_populated_fields, field_sets),
		'ordered_objects': ordered_objects,
		'inline_related_objects': inline_related_objects,
		'form_url': '',
		'opts': opts,
		'content_type_id': ContentType.objects.get_for_model(model).id,
	}
	c.update(extra_context)
	return render_to_response("admin/view.html", context_instance=c)



class CSVForm(forms.Form):
	delimiter = forms.ChoiceField(choices = [(';', ';'),
											 (',', ','),
											 ('|', '|')])
	encoding = forms.ChoiceField(choices = [('cp1252', 'Windows'),
											('mac_roman', 'Mac'),
											('latin1', 'Latin1'),
											('utf8', 'UTF-8') ])
	import_mode = forms.ChoiceField(choices = [('change', _('Add or replace')),
											   ('add', _('Add or pass')),
											   ('test', _('Test')),
											   ('delete', _('Delete'))],
									 widget = forms.RadioSelect)
	skip_first = forms.BooleanField()
								  

								  
								
def csv_form(request, app_label, model_name):
	"CSV Import/Export function"
	model = models.get_model(app_label, model_name)
	if model is None:
		raise Http404("App %r, model %r, not found" % (app_label, model_name))
	opts = model._meta
	field_sets = opts.admin.get_field_sets(opts)
	fields = []
	for field_set in field_sets:
		for field_line in field_set.field_lines:
			for field in field_line.fields:
				if not request.POST or request.POST.has_key(field.name):
					field = copy.copy(field)
					field.checked = 'checked'
				fields.append(field)

	output = []
	if request.POST:
		form = CSVForm(request.POST)
		encoding = request.POST['encoding']
		action = request.POST['action']
		import_mode = request.POST['import_mode']
		delimiter = request.POST['delimiter']
		skip_first = request.POST.get('skip_first', False)
	else:
		form = CSVForm(initial = { 'delimiter' : ':',
								   'encoding' : 'latin1',
								   'import_mode' : 'change',
								   'skip_first' : True})
		
	input = []
	if request.FILES:
		csv_data = request.FILES['csv']['content']
		if csv_data.find('\r\n') == -1 and csv_data.find('\r') != -1:
			csv_data = csv_data.replace('\r', '\n') # mac2unix
		output, input = csv_import(csv_data, encoding, delimiter, import_mode, request.POST, app_label, model_name, skip_first)

	elif request.POST and action == 'export':
		return csv_export(request, app_label, model_name, encoding, delimiter)
		
	
	return render_to_response('admin/import_form.html',
							  { 'title' : _('CSV Import / Export'),
								'fields' : fields,
								'output' : output,
								'input' : input,
								'form' : form,
								'cl' : ChangeList(request, model),
								'model_name' : model.__name__
								},
							  context_instance=RequestContext(request))




def csv_export(request, app_label, model_name, encoding, delimiter):
	model = models.get_model(app_label, model_name)
	opts = model._meta

	max_show_all_allowed = main.MAX_SHOW_ALL_ALLOWED
	main.MAX_SHOW_ALL_ALLOWED = 1000000
	cl = main.ChangeList(request, model)
	cl.show_all = True
	field_sets = opts.admin.get_field_sets(opts)

	keys = []
	for field_set in field_sets:
		for field_line in field_set.field_lines:
			for field in field_line.fields:
				if request.POST.has_key(field.name):
					keys.append(field.name)

	response = HttpResponse()
	response['Content-Type'] = "text/csv; charset=%s" % encoding
	response['Content-Disposition'] = 'attachment; filename="contacts.csv"'
	out = csv.writer(response, delimiter = delimiter)
	out.writerow(keys)
	for ob in cl.result_list:
		row = []
		for key in keys:
			value = str(getattr(ob, key))
			value = value != 'None' and unicode(value, 'utf8', errors='replace').encode(encoding, 'replace') or ''
			row.append(value)
		out.writerow(row)
	main.MAX_SHOW_ALL_ALLOWED = max_show_all_allowed
	return response





def csv_import(csv_data, encoding, delimiter, import_mode, form, app_label, model_name, skip_first):
	model = models.get_model(app_label, model_name)
	if model is None:
		raise Http404("App %r, model %r, not found" % (app_label, model_name))

	field_sets = model._meta.admin.get_field_sets(model._meta)

	csv_data = cStringIO.StringIO(csv_data)
	csv_reader = csv.reader(utf8_encoder(csv_data, encoding), delimiter = delimiter)
	message = []
	test_message = []
	if not form:
		csv_fields = csv_reader.next()
	else:
		csv_fields = form.keys()

	fields = []
	mandatory_fields = []
	unique_fields = []
	for field_set in field_sets:
		for field_line in field_set.field_lines:
			for field in field_line.fields:
				if field.name in csv_fields:
					fields.append(field)
					if not field.null:
						mandatory_fields.append(field)
					if field.unique:
						unique_fields.append(field)
						
	if import_mode == 'test':
		test_message = [ [field.name for field in fields] ]

	dict_entities = {}
	for field in fields:
		if field.rel and not field.rel.raw_id_admin:
			entity = field.rel.to
			primary_key_field = field.rel.field_name
			dict_entities[entity] = {}
			for e in entity.objects.all():
				dict_entities[entity][str(e)] = getattr(e, primary_key_field)
			
	nbcols = len(fields)
	n = 0
	for row in csv_reader:
		n += 1
		if skip_first and n == 1:
			continue
		dict_row = {}
		dict_unique_fields = {}
		i = 0
		if len(row) != nbcols:
			message.append("ERROR : %d columns on line %d instead of %d : %s" % (len(row), n, nbcols, `row`))
			break
						   
		for cell in row:
			field = fields[i]
			if field.rel:
				if not field.rel.raw_id_admin:
					cell = dict_entities[field.rel.to].get(cell, None)
				dict_row[field.name + '_id'] = cell
			else:
				dict_row[field.name] = cell
			i += 1

		for field in unique_fields:
			dict_unique_fields[field.name] = dict_row[field.name]
		if '' in dict_unique_fields.values():
			message.append("Unique fields not filled for this row : (pass)\n%s" % `row`)
			continue

		objects = model.objects.filter(**dict_unique_fields)
		if len(objects) > 1:
			message.append("Too many (%s) objects for %s.objects.filter(%s) : %s" % \
						   (len(objects), model_name, `dict_unique_fields`))
			break
		if import_mode == 'test':
			test_message.append(row)
			j = 0
			test_row = []
			for field in fields:
				cell = "%s" % dict_row.get(field.name, dict_row.get(field.name+'_id'))
				if cell != row[j]:
					cell = "%s (%s)" % (cell, row[j])
				test_row.append(cell)
				j += 1
			test_message.append(test_row)
			
		if import_mode == 'delete':
			if objects:
				object = objects[0]
				object.delete()
			else:
				message.append("%s not found" % `dict_unique_fields`)
			continue
		if objects:
			object = objects[0]
			if import_mode == 'add':
				message.append("%s already exists" % `dict_unique_fields`)
				continue
			else:
				for key, value in dict_row.items():
					if value:
						setattr(object, key, value)
		else:
			object = model(**dict_row)
		if import_mode in ('add', 'change'):
			safe_save(object)
			object.save()
	else:
		transaction.commit()
		message.append('%d lines succesdfully procedeed' % n)
		return (message, test_message)
	transaction.rollback()
	message.append('%d lines procedeed with error (no change).' % n)
	return (message, test_message)


csv_import = transaction.commit_manually(csv_import)


def utf8_encoder(csv_data, encoding):
	for line in csv_data:
		yield unicode(line, encoding).encode('utf8', 'replace')




def stat_form(request, app_label, model_name):
	"Statistical function"
	stat_min = 2
	model = models.get_model(app_label, model_name)
	if model is None:
		raise Http404("App %r, model %r, not found" % (app_label, model_name))
	opts = model._meta
	field_sets = opts.admin.get_field_sets(opts)
	fields = []
	for field_set in field_sets:
		for field_line in field_set.field_lines:
			for field in field_line.fields:
				if not request.POST or request.POST.has_key(field.name):
					field = copy.copy(field)
					field.checked = 'checked'
				fields.append(field)

	stat_fields = []
	if request.POST:
		stat_min = int(request.POST.get('stat_min', "0"))
		table = model._meta.db_table
		curs = connection.cursor()

		curs.execute("select count(*) from %s" % table)
		n = curs.fetchone()[0]


		for field in fields:
			if request.POST.has_key(field.name):
				stat_field = []
				field = field.db_column or field.name
				stat_min_dict = { 'value' : "<b>%s</b" % _('Others'), 'count' : 0, 'percentage' : 0 }
				curs.execute("SELECT %s, count(*) as c FROM %s GROUP BY %s ORDER BY c DESC" % (field, table, field))
				for value, count in curs.fetchall():
					if not stat_min or count>stat_min:
						stat_field.append({'value' : value, 'count' : count, 'percentage' : count*100.0/n })
					elif count<=stat_min:
						stat_min_dict['count'] += count
				if stat_min and stat_min_dict['count']:
					stat_min_dict['percentage'] = stat_min_dict['count']*100.0/n
					stat_field.append(stat_min_dict)
				stat_fields.append({'name':field, 'stats' : stat_field})
		
	
	return render_to_response('admin/stat_form.html',
							  { 'title' : _('Statistics'),
								'fields' : fields,
								'stat_fields' : stat_fields,
								'stat_min' : stat_min,
								'model_name' : model.__name__
								},
							  context_instance=RequestContext(request))
