from django.utils.cache import patch_vary_headers
from django.utils import translation
from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponse

import django.pimentech.log as log

import re

class LocaleHostNameMiddleware:
	regex = re.compile(r"^(\w\w)\.")

	def get_language_from_request (self,request):
		supported = dict(settings.LANGUAGES)
		lang = None
		check = self.regex.match(request.META['HTTP_HOST'])
		if check is not None:
			t = check.group(1)
			if t in supported:
				lang = (t == 'uk') and 'en' or t
		return lang

	def process_request(self, request):
		language = self.get_language_from_request(request) or settings.LANGUAGE_CODE[:2]
		translation.activate(language)
		request.LANGUAGE_CODE = translation.get_language()

	def process_response(self, request, response):
		patch_vary_headers(response, ('Accept-Language',))
		language = translation.get_language()
		response['Content-Language'] = language
		translation.deactivate()
		return response
